@@ -2,7 +2,7 @@ package RedisDB;
 
 use strict;
 use warnings;
-our $VERSION = "2.35";
+our $VERSION = "2.36";
 $VERSION = eval $VERSION;
 
 use RedisDB::Error;
@@ -205,7 +205,7 @@ sub _on_disconnect {
     if ($err) {
         $error_obj ||= RedisDB::Error::DISCONNECTED->new(
             "Server unexpectedly closed connection. Some data might have been lost.");
-        if ( $self->{raise_error} || $self->{_in_multi} ) {
+        if ( $self->{raise_error} or $self->{_in_multi} or $self->{_watching} ) {
             $self->reset_connection;
             die $error_obj;
         }
@@ -413,7 +413,7 @@ sub _recv_data_nb {
         else {
             delete $self->{_socket};
 
-            if ( $self->{_parser}->callbacks or $self->{_in_multi} ) {
+            if ( $self->{_parser}->callbacks or $self->{_in_multi} or $self->{_watching} ) {
 
                 # there are some replies lost
                 $self->_on_disconnect(1);
@@ -690,7 +690,7 @@ sub get_reply {
 
     my $res = shift @{ $self->{_replies} };
     if ( ref $res eq 'RedisDB::Error'
-        and ( $self->{raise_error} or $self->{_in_multi} ) )
+        and ( $self->{raise_error} or $self->{_in_multi} or $self->{_watching} ) )
     {
         croak $res;
     }
@@ -810,7 +810,7 @@ my @commands = qw(
   script_load   sdiff	sdiffstore	select	set
   setbit	setex	setnx	setrange	sinter	sinterstore
   sismember	slaveof	slowlog smembers	smove	sort	spop	srandmember
-  srem	sscan	strlen	sunion	sunionstore	sync	time    ttl	type	unwatch watch
+  srem	sscan	strlen	sunion	sunionstore	sync	time    ttl	type
   zadd	zcard	zcount	zincrby	zinterstore	zlexcount	zrange	zrangebylex
   zrangebyscore	zrank	zrem	zremrangebylex
   zremrangebyrank   zremrangebyscore	zrevrange	zrevrangebyscore	zrevrank
@@ -1086,9 +1086,10 @@ establish a new connection. If L</raise_error> disabled, the module will pass
 L<RedisDB::Error::DISCONNECTED> object to all outstanding callbacks and will
 try to reconnect to the server; it will also automatically restore
 subscriptions if object was in subscription mode. Module never tries to
-reconnect after MULTI command was sent to server and before corresponding EXEC
-or DISCARD was sent as this may cause data corruption, so during transaction
-module behaves like if L</raise_error> is set.
+reconnect after MULTI or WATCH command was sent to server and before
+corresponding UNWATCH, EXEC or DISCARD was sent as this may cause data
+corruption, so during transaction module behaves like if L</raise_error> is
+set.
 
 Module makes several attempts to reconnect each time increasing interval before
 the next attempt, depending on the values of L</reconnect_attempts> and
@@ -1457,6 +1458,44 @@ exec will return false value.
 
 =cut
 
+=head2 $self->watch(@keys[, \&callback])
+
+mark given keys to be watched
+
+=cut
+
+sub watch {
+    my $self = shift;
+
+    $self->{_watching} = 1;
+    if ( ref $_[-1] eq 'CODE' ) {
+        return $self->send_command( 'WATCH', @_ );
+    }
+    else {
+        return $self->execute( 'WATCH', @_ );
+    }
+}
+
+=head2 $self->unwatch([\&callback])
+
+unwatch all keys
+
+=cut
+
+sub unwatch {
+    my $self = shift;
+
+    my $res;
+    if ( ref $_[-1] eq 'CODE' ) {
+        $res = $self->send_command( 'UNWATCH', @_ );
+    }
+    else {
+        $res = $self->execute( 'UNWATCH', @_ );
+    }
+    $self->{_watching} = undef;
+    return $res;
+}
+
 =head2 $self->multi([\&callback])
 
 Enter the transaction. After this and till I<exec> or I<discard> will be called,
@@ -1497,6 +1536,7 @@ sub exec {
         $res = $self->execute('EXEC');
     }
     $self->{_in_multi} = undef;
+    $self->{_watching} = undef;
     return $res;
 }
 
@@ -1517,6 +1557,7 @@ sub discard {
         $res = $self->execute('DISCARD');
     }
     $self->{_in_multi} = undef;
+    $self->{_watching} = undef;
     return $res;
 }
 
